Package: FSA
Version: 0.9.5
Date: 2023-8-25
Title: Simple Fisheries Stock Assessment Methods
Description: A variety of simple fish stock assessment methods.
Authors@R: c(
    person(c("Derek","H."), "Ogle", email="DerekOgle51@gmail.com",
           role=c("aut","cre"),
           comment=c(ORCID="0000-0002-0370-9299")),
    person(c("Jason","C."), "Doll", email="jason.doll@fmarion.edu",
           role=c("aut")),
    person(c("A.","Powell"), "Wheeler", email="powell.wheeler@gmail.com",
           role="aut"),
    person("Alexis", "Dinno", email="alexis.dinno@pdx.edu",
           role="aut",
           comment="Provided base functionality of dunnTest()")
    )
URL: https://fishr-core-team.github.io/FSA/
BugReports: https://github.com/fishR-Core-Team/FSA/issues
License: GPL (>= 2)
LazyData: true
Depends: R (>= 3.5.0)
Imports: graphics, grDevices, stats, tools, utils, car, dunn.test,
        lmtest, plotrix, withr
Suggests: DescTools, dplyr, fishmethods, FSAdata, knitr, marked, nlme,
        nlstools, pkgdown, plyr, psych, Rcapture, rmarkdown, testthat,
        tibble, covr
Encoding: UTF-8
RoxygenNote: 7.2.3
NeedsCompilation: no
Packaged: 2023-08-26 18:16:51 UTC; derek
Author: Derek H. Ogle [aut, cre] (<https://orcid.org/0000-0002-0370-9299>),
  Jason C. Doll [aut],
  A. Powell Wheeler [aut],
  Alexis Dinno [aut] (Provided base functionality of dunnTest())
Maintainer: Derek H. Ogle <DerekOgle51@gmail.com>
Repository: CRAN
Date/Publication: 2023-08-26 18:50:02 UTC
Built: R 4.4.0; ; 2024-05-05 17:07:09 UTC; unix
