# Generated by roxygen2: do not edit by hand

S3method(Summarize,default)
S3method(Summarize,formula)
S3method(anova,catchCurve)
S3method(anova,depletion)
S3method(catchCurve,default)
S3method(catchCurve,formula)
S3method(chapmanRobson,default)
S3method(chapmanRobson,formula)
S3method(coef,catchCurve)
S3method(coef,chapmanRobson)
S3method(coef,depletion)
S3method(confint,boot)
S3method(confint,catchCurve)
S3method(confint,chapmanRobson)
S3method(confint,depletion)
S3method(confint,mrClosed1)
S3method(confint,mrClosed2)
S3method(confint,mrOpen)
S3method(confint,nlsBoot)
S3method(confint,removal)
S3method(dunnTest,default)
S3method(dunnTest,formula)
S3method(hist,boot)
S3method(hist,formula)
S3method(histFromSum,default)
S3method(histFromSum,formula)
S3method(histFromSum,table)
S3method(htest,boot)
S3method(htest,nlsBoot)
S3method(ksTest,default)
S3method(ksTest,formula)
S3method(lencat,default)
S3method(lencat,formula)
S3method(plot,CapHist)
S3method(plot,ageBias)
S3method(plot,catchCurve)
S3method(plot,chapmanRobson)
S3method(plot,depletion)
S3method(plot,mrClosed2)
S3method(predict,boot)
S3method(predict,nlsBoot)
S3method(print,dunnTest)
S3method(print,extraTest)
S3method(print,metaM)
S3method(psdAdd,default)
S3method(psdAdd,formula)
S3method(rSquared,catchCurve)
S3method(rSquared,default)
S3method(rSquared,depletion)
S3method(rSquared,lm)
S3method(sumTable,formula)
S3method(summary,ageBias)
S3method(summary,agePrec)
S3method(summary,catchCurve)
S3method(summary,chapmanRobson)
S3method(summary,depletion)
S3method(summary,mrClosed1)
S3method(summary,mrClosed2)
S3method(summary,mrOpen)
S3method(summary,removal)
S3method(wrAdd,default)
S3method(wrAdd,formula)
export(GompertzFuns)
export(Mmethods)
export(RichardsFuns)
export(Schnute)
export(SchnuteRichards)
export(Subset)
export(Summarize)
export(addZeroCatch)
export(ageBias)
export(agePrecision)
export(alkAgeDist)
export(alkIndivAge)
export(alkMeanVar)
export(alkPlot)
export(binCI)
export(bootCase)
export(capFirst)
export(capHistConvert)
export(capHistSum)
export(catchCurve)
export(chapmanRobson)
export(chooseColors)
export(col2rgbt)
export(compIntercepts)
export(compSlopes)
export(depletion)
export(diags)
export(dunnTest)
export(expandCounts)
export(expandLenFreq)
export(extraSS)
export(fact2num)
export(filterD)
export(fishR)
export(fitPlot)
export(fsaNews)
export(geomean)
export(geosd)
export(growthFunShow)
export(headtail)
export(histFromSum)
export(hoCoef)
export(htest)
export(hyperCI)
export(is.CapHist)
export(is.even)
export(is.odd)
export(jolly)
export(kCounts)
export(kPvalue)
export(ksTest)
export(lagratio)
export(lencat)
export(logbtcf)
export(logisticFuns)
export(lrt)
export(lwCompPreds)
export(mapvalues)
export(metaM)
export(mrClosed)
export(mrOpen)
export(nlsTracePlot)
export(pcumsum)
export(peek)
export(perc)
export(plotAB)
export(plotBinResp)
export(poiCI)
export(psdAdd)
export(psdCI)
export(psdCalc)
export(psdPlot)
export(psdVal)
export(purl2)
export(rSquared)
export(rcumsum)
export(removal)
export(repeatedRows2Keep)
export(reproInfo)
export(residPlot)
export(se)
export(srFunShow)
export(srFuns)
export(srStarts)
export(sumTable)
export(tictactoe)
export(validn)
export(vbFuns)
export(vbStarts)
export(wrAdd)
export(wsVal)
